package net.http {
	
	import flash.net.URLRequest;
	import flash.net.URLRequestHeader;
	import flash.net.URLRequestMethod;
	import flash.net.URLVariables;
	import flash.utils.ByteArray;
	import flash.utils.Endian;
	import net.http.requestfield.File;
	import net.http.requestfield.IByteableField;
	import net.http.requestfield.Parameter;
	
	/**
	 * Класс для формирования запросов на сервер 
	 * При необходимости (если отправляются файлы)
	 * заголовок content-type выставляется в значение multipart/form-data
	 * 
	 * @author gloomyBrain
	 */
	public class Request {
		
		/**
		 *  Символ новой строки
		 */
		public static const LB:String = "\r\n";
		
		/**
		 *  Символ двойного тире
		 */
		public static const DD:String = "--";
		
		/**
		 * URL-адрес запроса
		 */
		public var url:String;
		
		/**
		 * Метод запроса (POST или GET)
		 * Используются константы класса flash.net.URLRequestMethod
		 */
		public var method:String;
		
		private var _parameters:Object;
		
		private const _files:Vector.<File> = new Vector.<File>();
		private const _headers:Vector.<URLRequestHeader> = new Vector.<URLRequestHeader>();
		
		private static function getRandomBoundary():String {
			var result:String = "";
			for (var i:int = 0; i < 0x20; i++ ) {
				result += String.fromCharCode(int(97 + Math.random() * 25));
			}
			
			return result;
		}
		
		/**
		 * @param	url 	URL-адрес сервера, на который отправляется запрос
		 * @param 	params 	Объект, описывающий передаваемые переменные (пары имя-значение)
		 * @param 	method 	Метод запроса, по умолчанию POST (используются константы класса flash.net.URLRequestMethod)
		 */
		public function Request(url:String = "", params:Object = null, method:String = "POST") {
			
			this.url = url;
			this.method = method;
			
			_parameters = (params != null) ? params : {};
			
		}
		
		/**
		 * Объект параметров запроса (пары имя-значение)
		 */
		public function get parameters():Object {
			return _parameters;
		}
		
		/**
		 * Массив файлов, отправляемых в запросе
		 */
		public function get files():Vector.<File> {
			return _files;
		}
		
		/**
		 * Массив заголовков запроса
		 */
		public function get headers():Vector.<URLRequestHeader> {
			return _headers;
		}
		
		/**
		 * Формирует URLRequest со всеми необходимыми параметрами
		 */
		public function toURLRequest():URLRequest {
			
			var result:URLRequest = (_files.length == 0) ? this.formCommonRequest() : this.formMultipartRequest();
			
			var i:int = 0;
			var len:int = _headers.length;
			
			for (i = 0; i < len; i++) {
				result.requestHeaders.push(_headers[i]);
			}
			
			return result;
			
		}
		
		/**
		 * Формирует массив параметров запроса
		 */
		protected function formParamsArray():Array/*of Parameter*/{
			
			var result:Array/*of Parameter*/= [];
			
			for (var name:String in _parameters) {
				result.push(new Parameter(name, _parameters[name]));
			}
			
			return result;
		}
		
		/**
		 * @private
		 */
		private function formCommonRequest():URLRequest {
			
			var result:URLRequest = new URLRequest(this.url);
			var paramsArray:Array/*of Parameter*/= this.formParamsArray();
			var data:URLVariables = new URLVariables();
			var len:int = paramsArray.length;
			var current:Parameter = null;
			
			while (len--) {
				current = paramsArray[len] as Parameter;
				data[current.name] = current.value;
			}
			
			result.method = this.method;
			result.data = data;
			
			return result;
		}
		
		/**
		 * @private
		 */
		private function formMultipartRequest():URLRequest {
			
			var result:URLRequest = new URLRequest(this.url);
			var paramsArray:Array/*of IByteableField*/ = this.formParamsArray();
			var boundary:String = Request.getRandomBoundary();
			var data:ByteArray = new ByteArray();
			var len:int = 0;
			
			data.endian = Endian.BIG_ENDIAN;
			
			len = _files.length;
			while (len--) {
				paramsArray.push(_files[len]);
			}
			
			len = paramsArray.length;
			while (len--) {
				data.writeUTFBytes(Request.DD + boundary + Request.LB);
				paramsArray[len].toBytes(data);
			}
			
			data.writeUTFBytes(Request.DD + boundary + Request.DD + Request.LB);
			
			result.method = URLRequestMethod.POST;
			result.contentType = "multipart/form-data; boundary=\"" + boundary + "\"";
			result.data = data;
			
			return result;
		}
		
	}

}